################################################################################
#                                                                              #
#      :::[  T h e   R u s s i a n   E g g d r o p  R e s o u r c e  ]:::      #
#    ____                __                                                    #
#   / __/___ _ ___ _ ___/ /____ ___   ___      ___   ____ ___ _     ____ __ __ #
#  / _/ / _ `// _ `// _  // __// _ \ / _ \    / _ \ / __// _ `/    / __// // / #
# /___/ \_, / \_, / \_,_//_/   \___// .__/ __ \___//_/   \_, / __ /_/   \___/  #
#      /___/ /___/                 /_/    /_/           /___/ /_/              #
#                                                                              #
################################################################################
#                                                                              #
# commands.tcl 1.3                                                             #
#                                                                              #
# Author: Stream@Rusnet <stream@eggdrop.org.ru>                                #
#                                                                              #
# Official support: irc.eggdrop.org.ru @ #eggdrop                              #
#                                                                              #
################################################################################

namespace eval commands {}

setudef flag nopubcommands

#################################################

bind pub - !command	::commands::pub_commands
bind pub - !commands	::commands::pub_commands
bind pub - !	::commands::pub_commands
bind pub - !	::commands::pub_commands

bind msg - !command	::commands::msg_commands
bind msg - !commands	::commands::msg_commands
bind msg - !	::commands::msg_commands
bind msg - !	::commands::msg_commands

#################################################

foreach p [array names commands *] {catch {unset commands($p)}}

# help data file
set commands(file)	"commands.txt"
# maximum length of a message
set commands(maxlen)	400
# show commands that exist in [binds] only (recomended to enable this)
set commands(onlybinds)	1
# check users flags and show only allowed commands (only if previous setting == 1)
set commands(allowed)	1

set commands(debug)	1

#################################################
##### DON'T CHANGE ANYTHING BELOW THIS LINE #####
#################################################

set commands(data)	""

set commands(ver)	"1.3"
set commands(authors)	"Stream@RusNet <stream@eggdrop.org.ru>"

#################################################

proc ::commands::readdata {} {
	global commands
	set commands(data) ""
	set typ 0
	set cond ""
	set subj ""
	set data ""
	if {![catch {set fid [open $commands(file) "r"]}]} {
		while {![eof $fid]} {
			set str [string trim [gets $fid]]
			if {[string length $str] == 0 || [string range $str 0 0] == "#"} {continue}
			if {[string range $str 0 0] == "|"} {
				if {$subj != ""} {lappend commands(data) [list $typ $cond $subj $data]}
				set str [split [string range $str 1 end] "|"]
				set typ [lindex $str 0]
				set cond [lindex $str 1]
				set subj [lrange $str 2 end]
				set data ""
			} else {lappend data $str}
		}
		if {$subj != ""} {lappend commands(data) [list $typ $cond $subj $data]}
		close $fid
		putlog "\[commands\] loaded [llength $commands(data)] command descriptions"
	} else {putlog "\[commands\] can't open file '$commands(file)'!"}
}

proc ::commands::out {nick chan text} {
	global commands botnick
	set cmd ""
	if {$nick != $botnick} {
		if {$nick == $chan} {set cmd "PRIVMSG $nick :"
		} else {set cmd "NOTICE $nick :"}
	} else {return}
	regsub -all -nocase -- {\[b\]} $text {} text
	regsub -all -nocase -- {\[u\]} $text {} text
	regsub -all -nocase -- {\[c(\d){0,2}(,)(\d){0,2}\]} $text {\1\2\3} text
	regsub -all -nocase -- {\[c(\d){0,2}\]} $text {\1} text
	while {[string length $text] > 0} {
		if {[string length $text] <= $commands(maxlen) || [llength [split $text]] == 1} {putserv "$cmd$text"; break}
		set msg ""
		set newtext ""
		foreach word [split $text] {
			if {$newtext == "" && [expr [string length $msg] + [string length $word]] <= $commands(maxlen)} {lappend msg $word
			} elseif {$newtext == "" && $msg == "" && [string length $word] > $commands(maxlen)} {lappend msg $word
			} else {lappend newtext $word}
		}
		if {$msg != ""} {putserv "$cmd[join $msg]"}
		set text [join $newtext]
	}
}

proc ::commands::tolower {text} {
	return [string map {                                                                 } [string tolower $text]]
}

proc ::commands::bindexists {bnds text} {
	global commands
	if {$commands(onlybinds) == 0} {return 1}
	foreach cmd $bnds {if {[string equal [lindex $cmd 2] $text]} {return 1}}
	return 0
}

proc ::commands::allowed {nick hand chan bnds cmd} {
	global commands
	if {$commands(allowed) == 0 || $commands(onlybinds) == 0} {return 1}
	foreach bnd $bnds {
		if {[string equal [lindex $bnd 2] $cmd]} {
			if {[lindex $bnd 1] == "-|-"} {return 1}
			if {[validuser $hand]} {
				if {[validchan $chan]} {return [matchattr $hand [lindex $bnd 1] $chan]}
				return [matchattr $hand [lindex $bnd 1]]
			}
			break
		}
	}
	return 0
}

proc ::commands::init {} {
	::commands::readdata
}

#################################################

proc ::commands::pub_commands {nick uhost hand chan args} {
	if {[channel get $chan nopubcommands]} {return}
	set text [string trim [join $args]]
	regsub -all -nocase -- {\\\[\]\{\}} $text "" text
	::commands::show $nick $hand $chan [string trim $text]
}

proc ::commands::msg_commands {nick uhost hand args} {
	set text [string trim [join $args]]
	regsub -all -nocase -- {\\\[\]\{\}} $text "" text
	::commands::show $nick $hand $nick [string trim $text]
}

#################################################

proc ::commands::show {nick hand chan text} {
	global commands
	putlog "\[commands\] $nick/$chan $text"
	set bnds ""
	if {$commands(onlybinds) == 1} {
		foreach cmd [binds pub] {
			lappend bnds $cmd
		}
#		foreach cmd [binds msg] {
#			lappend bnds $cmd
#		}
	}
	if {$text == ""} {
		set res ""
		foreach data $commands(data) {
			set cond [string trim [lindex $data 1]]
			if {[string length $cond] != 0} {if {![eval $cond]} {continue}}
			set subj [lindex [lindex $data 2] 0]
			if {[lindex $data 0] < 2 && ([::commands::bindexists $bnds $subj] == 0 || [::commands::allowed $nick $hand $chan $bnds $subj] == 0)} {continue}
			if {[lindex $data 0] == 0 || [lindex $data 0] == 3} {lappend res $subj}
		}
		if {$res == ""} {::commands::out $nick $chan "    ."; return}
		set res [join $res "; "]
		::commands::out $nick $chan "   : $res"
		::commands::out $nick $chan "      \002!commands <>\002"
		return
	}
	set txt [::commands::tolower $text]
	foreach data $commands(data) {
		set cond [string trim [lindex $data 1]]
		if {[string length $cond] != 0} {if {![eval $cond]} {continue}}
		set found 0
		set allowed 0
		set subj [lindex $data 2]
		foreach alias $subj {
			if {[lindex $data 0] < 2 && [::commands::bindexists $bnds [lindex $subj 0]] == 0} {continue}
			if {[string equal $alias $txt]} {
				set found 1
				if {[lindex $data 0] > 1 || [::commands::allowed $nick $hand $chan $bnds [lindex $subj 0]] != 0} {
					set allowed 1
					break
				}
			}
		}
		if {$found == 1} {
			if {$allowed == 1} {
				if {[llength $subj] > 1} {
					set aliases [join [lrange $subj 1 end] ", "]
					::commands::out $nick $chan "\002[lindex $subj 0]\002 ((): \002$aliases\002):"
				} else {::commands::out $nick $chan "\002$subj\002:"}
				foreach line [lindex $data 3] {::commands::out $nick $chan "$line"}
			} else {::commands::out $nick $chan " '$text'  ."}
			break
		}
	}
	if {$found == 0} {::commands::out $nick $chan "   '$text' ."}
}

::commands::init
putlog "commands.tcl v$commands(ver) by $commands(authors) loaded"
